/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.apache.tuscany.core.implementation.composite;

import java.util.Map;

import org.w3c.dom.Document;

import org.apache.tuscany.spi.builder.Connector;
import org.apache.tuscany.spi.component.CompositeComponent;

/**
 * The standard implementation of a composite component. Autowiring is performed by delegating to the parent composite.
 *
 * @version $Rev: 451895 $ $Date: 2006-10-01 23:58:18 -0700 (Sun, 01 Oct 2006) $
 */
public class CompositeComponentImpl extends AbstractCompositeComponent {
    private String uri;

    public CompositeComponentImpl(String name,
                                  CompositeComponent parent,
                                  Connector connector,
                                  Map<String, Document> propertyValues) {
        this(name, null, parent, connector, propertyValues);
    }

    /**
     * Constructor specifying name and URI.
     *
     * @param name           the name of this Component
     * @param uri            the unique identifier for this component
     * @param parent         this component's parent
     * @param connector
     * @param propertyValues this composite's Property values
     */
    public CompositeComponentImpl(String name,
                                  String uri,
                                  CompositeComponent parent,
                                  Connector connector,
                                  Map<String, Document> propertyValues) {
        super(name, parent, connector, propertyValues);
        this.uri = uri;
    }

    public String getURI() {
        return uri;
    }

}
