/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.tuscany.core.implementation.composite;

import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.Map;

import org.apache.tuscany.spi.component.WorkContext;
import org.apache.tuscany.spi.model.Operation;
import org.apache.tuscany.spi.wire.InboundWire;
import org.apache.tuscany.spi.wire.Interceptor;
import org.apache.tuscany.spi.wire.Message;
import org.apache.tuscany.spi.wire.MessageImpl;
import org.apache.tuscany.spi.wire.OutboundInvocationChain;

import junit.framework.TestCase;
import org.easymock.EasyMock;

/**
 * @version $Rev: 476842 $ $Date: 2006-11-19 08:03:50 -0800 (Sun, 19 Nov 2006) $
 */
public class CompositeReferenceCallbackTargetInvokerInvocationExceptionTestCase extends TestCase {
    private InboundWire wire;
    private WorkContext context;
    private Message message;
    private OutboundInvocationChain chain;
    private CompositeReferenceCallbackTargetInvoker invoker;

    /**
     * Verfies an InvocationTargetException thrown when invoking the target is propagated to the client correctly and
     * the originating error is unwrapped
     */
    public void testThrowableTargetInvocation() throws Exception {
        Message response = invoker.invoke(message);
        assertTrue(response.isFault());
        Object body = response.getBody();
        assertTrue(SomeException.class.equals(body.getClass()));
        EasyMock.verify(wire);
        EasyMock.verify(context);
        EasyMock.verify(chain);
    }

    protected void setUp() throws Exception {
        super.setUp();
        Object id = new Object();
        Object corrId = new Object();
        Object targetAddress = new Object();
        message = new MessageImpl();
        message.setMessageId(id);
        message.setCorrelationId(corrId);
        message.setBody("foo");
        Message response = new MessageImpl();
        response.setBody("response");
        Operation<Type> operation = new Operation<Type>("echo", null, null, null);
        Interceptor head = new ErrorInterceptor();
        chain = EasyMock.createMock(OutboundInvocationChain.class);
        EasyMock.expect(chain.getTargetInvoker()).andReturn(null);
        EasyMock.expect(chain.getHeadInterceptor()).andReturn(head);
        EasyMock.replay(chain);
        Map<Operation<?>, OutboundInvocationChain> chains = new HashMap<Operation<?>, OutboundInvocationChain>();
        chains.put(operation, chain);
        wire = EasyMock.createMock(InboundWire.class);
        EasyMock.expect(wire.retrieveMapping(corrId)).andReturn(targetAddress);
        EasyMock.expect(wire.getSourceCallbackInvocationChains(targetAddress)).andReturn(chains);
        EasyMock.expect(wire.getContainer()).andReturn(null);
        EasyMock.replay(wire);
        context = EasyMock.createMock(WorkContext.class);
        context.setCurrentMessageId(EasyMock.eq(id));
        context.setCurrentMessageId(EasyMock.isNull());
        context.setCurrentCorrelationId(corrId);
        context.setCurrentCorrelationId(EasyMock.isNull());
        EasyMock.expect(context.getCurrentMessageId()).andReturn(id);
        EasyMock.expect(context.getCurrentCorrelationId()).andReturn(corrId);
        EasyMock.replay(context);
        invoker = new CompositeReferenceCallbackTargetInvoker(operation, wire, context);
    }

    private class SomeException extends Exception {

    }

    private class ErrorInterceptor implements Interceptor {

        public Message invoke(Message msg) {
            msg.setBodyWithFault(new SomeException());
            return msg;
        }

        public void setNext(Interceptor next) {

        }

        public Interceptor getNext() {
            return null;
        }

        public boolean isOptimizable() {
            return false;
        }
    }


}
