/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.apache.tuscany.spi.annotation;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.PARAMETER;
import java.lang.annotation.Retention;
import static java.lang.annotation.RetentionPolicy.RUNTIME;
import java.lang.annotation.Target;

/**
 * A system annotation to inject an autowired instance
 *
 * @version $Rev: 439728 $ $Date: 2006-09-02 23:02:44 -0700 (Sat, 02 Sep 2006) $
 */
@Target({METHOD, FIELD, PARAMETER})
@Retention(RUNTIME)
public @interface Autowire {
    /**
     * The name of the autowire. If not specified then the name will be derived from the annotated field.
     */
    String name() default "";
    
    /**
     * Indicates if a reference must be specified.
     */
    boolean required() default true;

}
