/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.apache.tuscany.spi.component;

/**
 * Implementations track information associated with a request as it is processed by the runtime
 *
 * @version $Rev: 441743 $ $Date: 2006-09-08 22:05:09 -0700 (Fri, 08 Sep 2006) $
 */
public interface WorkContext {

    Object getCurrentMessageId();

    void setCurrentMessageId(Object messageId);

    Object getCurrentCorrelationId();

    void setCurrentCorrelationId(Object correlationId);

    /**
     * Returns the composite where a remote request came in
     */
    CompositeComponent getRemoteComponent();

    /**
     * Sets the composite where a remote request came in
     */
    void setRemoteComponent(CompositeComponent component);

    /**
     * Returns the unique key for the given identifier associated with the current request
     */
    Object getIdentifier(Object type);

    /**
     * Sets the unique key for the given identifier associated with the current request
     */
    void setIdentifier(Object type, Object identifier);

    /**
     * Clears the unique key for the given identifier associated with the current request
     */
    void clearIdentifier(Object type);

    /**
     * Clears all identifiers associated with the current request
     */
    void clearIdentifiers();

}
