/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.apache.tuscany.container.groovy;

import java.lang.reflect.InvocationTargetException;

import org.apache.tuscany.spi.wire.InvocationRuntimeException;
import org.apache.tuscany.spi.wire.Message;
import org.apache.tuscany.spi.wire.TargetInvoker;

import groovy.lang.GroovyObject;

/**
 * Dispatches to a Groovy implementation instance
 *
 * @version $$Rev: 443468 $$ $$Date: 2006-09-14 13:56:03 -0700 (Thu, 14 Sep 2006) $$
 */
public class GroovyInvoker implements TargetInvoker, Cloneable {

    protected GroovyAtomicComponent component;
    protected String operation;
    protected boolean cacheable;

    public GroovyInvoker(String operation, GroovyAtomicComponent context) {
        this.component = context;
        this.operation = operation;
    }

    public boolean isCacheable() {
        return cacheable;
    }

    public void setCacheable(boolean cacheable) {
        this.cacheable = cacheable;
    }

    public boolean isOptimizable() {
        return false;
    }

    /**
     * Dispatches to the the target.
     */
    public Object invokeTarget(final Object payload) throws InvocationTargetException {
        GroovyObject target = component.getTargetInstance();
        Object[] args = (Object[]) payload;
        try {
            return target.invokeMethod(operation, args);
        } catch (Exception ex) {
            throw new InvocationTargetException(ex);
        }
    }

    public Message invoke(Message msg) throws InvocationRuntimeException {
        try {
            Object resp = invokeTarget(msg.getBody());
            msg.setBody(resp);
        } catch (InvocationTargetException e) {
            msg.setBodyWithFault(e.getCause());
        } catch (Throwable e) {
            msg.setBodyWithFault(e);
        }
        return msg;
    }

    public GroovyInvoker clone() throws CloneNotSupportedException {
        return (GroovyInvoker) super.clone();
    }


}
